#!/usr/bin/env python
################################################################################
#
# Copyright (c) Members of the EGEE Collaboration. 2004.
# See http://eu-egee.org/partners/ for details on the copyright holders.
# For license conditions see the license file or http://eu-egee.org/license.html
#
################################################################################
# glite-voms-config v. 3.0.4
#
# Post-installation script for configuring the gLite VOMS Server
# Robert Harakaly < robert.harakaly@cern.ch >
# Guillermo Diez-Andino <Guillermo.Sancho@cern.ch>
# Joachim Flammer <Joachim.Flammer@cern.ch>
#
# Version info: $Id: glite-voms-server-config.py,v 1.46.2.5 2006/05/16 08:26:59 flammer Exp $
#
# Usage: python glite-voms-server-config
#        --start             start the service
#        --stop              stop the service
#        --status            show the service status
#        --configure         configure the services
#
#If you only want to operate on a single VO without changing the configuration
#or status of the rest of the VOMS server you can specify in addition:
#
#    --vo=voName         operate only on VO 'voName'
# 
# The following commands only work with the --vo option:
#
#    --remove            remove VO 'voName'
#                        Option only works with voms-admin
#                        If the VO is running, it is stopped.
#                        You have to remove the configuration afterwards from the
#                        configuration files, otherwise the VO is added next time
#                        you run --configure
#    
# The following commands only work with the option --remove
# 
#    --remove-db         remove the database if it exists already
#                        Option only works with voms-admin installed
#                        Default is leaving the database alone
#
# Return codes: 0 - Ok
#               1 - Configuration failed
#
################################################################################

import sys, os, getopt, commands, pwd

import os,string,socket,pwd
import sys
import getopt
import time
import commands

from gLiteInstallerLib import ConfigParams
from gLiteInstallerLib import gLib

from mysql import Mysql
from tomcat5 import Secure_tomcat5

from gliteRgmaServicetool import gliteRgmaServicetool
from gliteRgmaServicetool import gliteRgmaServicetoolInstance

global rgmaServicetool

class glite_voms:

    #################################################################
    # init 
    #
    # init service class
    #
    #################################################################
    def __init__(self, params):
        
        self.verbose = 0
        self.version = "3.0.4"
        self.name = "glite-voms-server"
        self.friendly_name = "gLite VOMS Server"

        self.serviceName = "VOMS administration server for VO "        
        self.serviceType = "org.glite.security.voms"
        self.serviceStatusScript = "%s/etc/config/scripts/gLiteCheckWebserviceStatus.py" %  os.environ['GLITE_LOCATION']
        self.serviceVoEndpoint = "/voms/"
        self.serviceEndpointBase = "https://%s:8443%s" % (os.environ['HOSTNAME'], self.serviceVoEndpoint)
        
        # wsdl is base1/voName/base2
        self.serviceWsdlUrlBase1 = "https://%s:8443/voms-admin/" % os.environ['HOSTNAME']
        self.serviceWsdlUrlBase2 = "/services/VOMSAdmin?wsdl"
        
        self.params = params
        self.mysql = Mysql()
        self.tomcat = Secure_tomcat5(params)
        self.glib = params['glib']

    #################################################################
    # banner 
    #
    # show banner
    #
    #################################################################
    def banner(self):

        print "\nxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
        print "          Configuring the %s" % self.friendly_name
        print "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx\n"
    

    #################################################################
    # copyright 
    #
    # show copyright
    #
    #################################################################
    def copyright(self):

        print '\nCopyright (c) Members of the EGEE Collaboration. 2004'
        print 'See http://eu-egee.org/partners/ for details on the copyright holders'
        print 'For license conditions see the license file or http://eu-egee.org/license.html'


    #################################################################
    # showVersion 
    #
    # show version
    #
    #################################################################
    def showVersion(self):

        print '\n%s-config  v. %s\n' % (self.name,self.version)
    

    #################################################################
    # usage 
    #
    # print usage
    #
    #################################################################
    def usage(self,msg = ""):

        if msg:
            print "\n%s" % (msg)
        
        self.copyright()
        self.showVersion()
    
        print "Usage:"
        print
        print " Edit the configuration file %s.cfg.xml in %s/etc/config/templates" % (self.name, os.environ['GLITE_LOCATION'])
        print " save it as %s/etc/config/%s.cfg.xml and run the script as follows" % (os.environ['GLITE_LOCATION'], self.name)
        print " python %s-config [OPTION...]" % self.name
        
        print
        print '    -c, --checkconf     print the service configuration'
        print '    -v, --version       print the version of the configuration script'
        print '    -h, --help          print this usage information'
        print '    --verbose           enable verbosity' 
        print
        print '    --start             start the service'
        print '    --stop              stop the service'
        print '    --status            print service status (0 = running, 1 = idle)'
        print '    --configure         configure the service'
        print '    --siteconfig=URL    URL of the site configuration file'
        print
        print " If you only want to operate on a single VO without changing the configuration"
        print " or status of the rest of the VOMS server you can specify in addition:"
        print
        print "    --vo=voName         operate only on VO 'voName'"
        print 
        print " The following commands only work with the --vo option:"
        print
        print "    --remove            remove VO 'voName "
        print "                        Option only works with voms-admin"
        print "                        If the VO is running, it is stopped."
        print "                        You have to remove the configuration afterwards from the"
        print "                        configuration files, otherwise the VO is added next time"
        print "                        you run --configure"
        print 
        print " The following commands only work with the option --remove"
        print
        print "    --remove-db         remove the database if it exists already"
        print "                        Option only works with voms-admin installed"
        print "                        Default is leaving the database alone"
        print '\n'


    #################################################################
    # configure
    #
    # configure voms server
    #
    # return 0 ok
    #        1 error
    #
    #################################################################
    def configure(self):
                                
        # Decide on database type: MySQL or ORACLE (user)   
        if params.has_key('voms.db.type'):        
            if params['voms.db.type'] == 'mysql':
                self.db_type = "mysql"
            elif params['voms.db.type'] == 'oracle':
                self.db_type = "oracle"            
            else:
                # Incorrect parameter given
                glib.printFailedMessage()
                glib.printErrorMessage("Incorrect parameter 'voms.db.type' given.")
                glib.printErrorMessage("Must be 'mysql|oracle'. (Given parameter is '%s')" % params['voms.db.type'])
                return 1
        else:
            glib.printFailedMessage()
            glib.printErrorMessage("Parameter \"voms.db.type\" not found for.")
            return 1
        
        #--------------------------------------------------------------
        # Check Oracle instanclient libs
        #--------------------------------------------------------------
        if self.db_type == "oracle":
            if params.has_key('voms.db.oracle.instantclient.location'):
                if params['voms.db.oracle.instantclient.location'] != "":
                    if not os.path.exists(params['voms.db.oracle.instantclient.location']):
                        glib.printErrorMessage("The Oracle InstantClient cannot be found at the specified location")
                        glib.printErrorMessage(params['voms.db.oracle.instantclient.location'])
                        glib.printErrorMessage("Configuration cannot continue")
                        return 1

        #--------------------------------------------------------
        # Stop services
        #--------------------------------------------------------
        if self.stop() != 0:
            return 1
        
        #--------------------------------------------------------
        # Configure the Security Utilities
        #--------------------------------------------------------        
        if os.system("python %s/glite-security-utils-config.py --subservice" % self.glib.getScriptPath()):
            print "\nConfiguring gLite Security Utilities                     ",
            self.glib.printFailedMessage()
            return 1

        print "\nConfiguring gLite Security Utilities                      ",
        self.glib.printOkMessage()

        #--------------------------------------------------------
        # Get and check configuration parameters
        #--------------------------------------------------------
        
        print "Checking the common configuration values                  ",        

        # Decide on: install voms only or voms + voms-admin (user)
        if params.has_key('voms.admin.install'):
            self.voms_admin_install = params['voms.admin.install']
            if (self.voms_admin_install != "true") and (self.voms_admin_install != "false"):
                glib.printFailedMessage()
                glib.printErrorMessage("Incorrect parameter 'voms.admin.install' given.")
                glib.printErrorMessage("Must be 'true|false'. (Given parameter is '%s')" % self.voms_admin_install)
                return 1            
        else:
            glib.printFailedMessage()
            glib.printErrorMessage("Missing parameter: \"voms.admin.install\" not found.")
            return 1                                  

        # activate rgma servicetool (advanced)
        self.rgma_servicetool_activate = params['rgma.servicetool.activate']
        if (self.rgma_servicetool_activate != "true") and (self.rgma_servicetool_activate != "false"):
            glib.printFailedMessage()
            glib.printErrorMessage("Incorrect parameter 'rgma.servicetool.activate' given.")
            glib.printErrorMessage("Must be 'true|false'. (Given parameter is '%s')" % self.rgma_servicetool_activate)
            return 1            

        # MySQL configuration params
        if self.db_type == "mysql":

            # fix for bug no
            # max connections for mysql (advanced)
            self.mysql_maxConnections = params['voms.db.mysql.maxConnections']
            
        # Oracle configuration params
        if self.db_type == "oracle":
            
            # location for oracle instants client (advanced)
            self.oracle_instantclient_location = params['voms.db.oracle.instantclient.location']        

        glib.printOkMessage()

        #--------------------------------------------------------
        # Get R-GMA settings
        #--------------------------------------------------------
        if self.rgma_servicetool_activate=="true":
     
            # we create the R_GMA user here - this is normally done by the 
            # R-GMA servicetool but we need the home directory of it already
            if rgmaServicetool.configureRgmaUser(glib):
                return 1

            # rgma user name (system)
            rgmaUserName = params['rgma.user.name']

            # certificate path (system)
            userCertificatePath = params['user.certificate.path']

            # derive rgma user key and certificate path
            rgmaUserHomeDir     = pwd.getpwnam(rgmaUserName)[5] 
            self.rgmaCertificatePath = "%s/%s/hostcert.pem" % (rgmaUserHomeDir, userCertificatePath)
            self.rgmaKeyPath         = "%s/%s/hostkey.pem"  % (rgmaUserHomeDir, userCertificatePath)
            
        #--------------------------------------------------------
        # Check for JAVA
        #--------------------------------------------------------        
        if glib.checkJava() != 0:
            return 1
            
        #--------------------------------------------------------        
        # Check for the certificates
        #--------------------------------------------------------        
        if params.has_key('glite.installer.checkcerts'):
            if params['glite.installer.checkcerts'] == "true":
                if glib.check_certs(params) != 0:
                    return 1    
                
        #--------------------------------------------------------        
        # Configure Tomcat
        #--------------------------------------------------------        
        if self.tomcat.configure() != 0:
            return 1

        #--------------------------------------------------------        
        # Configure MySQL
        #--------------------------------------------------------                
        if self.db_type == "mysql":

            # -----------------------------------------
            # Update MySQL configuration if necessary
            # -----------------------------------------                
            if self.mysql.checkConfiguration("mysqld", "max_connections", self.mysql_maxConnections) !=0:
                print "Updating MySQL configuration ...                          "
            
                if self.mysql.setConfiguration("mysqld", "max_connections", self.mysql_maxConnections):
                    print "Updating MySQL configuration                              ",
                    glib.printFailedMessage()
                    return 1
            
                # -----------------------------------------
                # fix for bug no 15449: restart MySQL as the paramter has changed
                # -----------------------------------------                
                if self.mysql.restart():
                    print "Updating MySQL configuration                              "                    
                    glib.printFailedMessage()
                    return 1
                
                print "Updating MySQL configuration                              "                
                glib.printOkMessage()
        
        #--------------------------------------------------------        
        # Configure Oracle
        #--------------------------------------------------------                
        if self.db_type == "oracle":

            #--------------------------------------------------------------
            # Check Oracle instanclient libs
            #--------------------------------------------------------------            
            if not os.path.exists(self.oracle_instantclient_location):
                glib.printErrorMessage("The Oracle InstantClient cannot be found at the specified location")
                glib.printErrorMessage(self.oracle_instantclient_location)
                glib.printErrorMessage("Please check if you have installed the instant clients.")
                glib.printErrorMessage("Configuration cannot continue.")
                return 1
                
        #--------------------------------------------------------        
        # Configure VO instances
        #--------------------------------------------------------        
        instances = self.glib.getInstances('voms')

        if len(instances) > 0:
            for instance in instances:
                retval = self.configureVomsInstance(instance)
                if retval == 1:
                    return 1
        else:
            self.configureVomsInstance()  
        
        #--------------------------------------------------------         
        # Fix for bug 10363 - set correctly the permission of the config directory
        #--------------------------------------------------------        
        os.chmod("%s/etc" % os.environ['GLITE_LOCATION_VAR'], 0755)
        
        #--------------------------------------------------------        
        # Create directory structure - is it still necessary ?
        #--------------------------------------------------------        
        glib.check_dir('%s/lock/subsys/' % self.params['GLITE_LOCATION_VAR'])
        
        #--------------------------------------------------------        
        # Add gLite setenv script to cron script (bug 14912)
        #--------------------------------------------------------        
        cronfile = "%s/etc/cron/cron-voms-ldap-sync" % os.environ['GLITE_LOCATION']
        cronfileTmp = "%s.tmp" % cronfile

        scriptLine = "source %s\n" % self.params['installer.export.filename']

        # 0: header not processed
        # 1: header processing
        # 2: after header
        headerFlag = 0
        
        try:                                        
            input = open(cronfile, 'r')
            output = open(cronfileTmp, 'w')
                
            inlines = input.readlines()
            for line in inlines:
                
                # check for comment line and filter out header
                if line.startswith("#"):
                    output.write(line)  # normal comment line => untouched
                    if headerFlag == 0: # before header: starting header processing
                        headerFlag = 1  # set header processing
                else:
                    if headerFlag == 1: # header processing done
                        # check if line does not exist yet
                        if line != scriptLine:
                            output.write(scriptLine)
                        output.write(line)
                        headerFlag = 2 # after header processing
                    else:
                        output.write(line)                            

            output.close()        
            input.close()
            os.rename(cronfileTmp, cronfile)
            os.chmod(cronfile,0755)
            
        except IOError, ex:
            glib.printFailedMessage()
            glib.printErrorMessage("\nFile '%s' is not writable" % cronfileTmp)
            return 1
        
        #--------------------------------------------------------
        # RGMA servicetool: configure servicetool
        #--------------------------------------------------------        
        print
        if self.rgma_servicetool_activate=="true":
            if rgmaServicetool.configure(self.glib):
                return 1
            
        # ------------------------------------------------------------
        # registering VOMS server
        # ------------------------------------------------------------
        
        glib.registerService()

        return 0


    #################################################################
    # configureVomsInstance
    #
    # configure one voms vo instance
    #
    # return 0 ok
    #        1 error
    #
    #################################################################
    def configureVomsInstance(self, name='default:default'):

        # extract vo name from 'default:voName' string
        vo_name_array = name.split(":")
        default_vo_name = vo_name_array[1]

        print "\nConfiguring VO '%s' ..." % default_vo_name        

        #--------------------------------------------------------
        # Get and check configuration parameters
        #--------------------------------------------------------
        
        print "Checking VO specific configuration values                 "
         
        try:
            iParams = self.glib.getInstanceParams(name)
        except ValueError, msg:
            print "Checking VO specific configuration values                 ",
            glib.printFailedMessage()
            glib.printErrorMessage("%s, Cannot continue" % msg)
            return 1
        except LookupError, msg:
            print "Checking VO specific configuration values                 ",            
            glib.printFailedMessage()
            glib.printErrorMessage(msg)
            return 1

        #--------------------------------------------------------
        # voms configuration values
        #--------------------------------------------------------

        # vo name - if not present: assign name of instance (user)
        if iParams.has_key('vo.name'):
            self.vo_name = iParams['vo.name']
        else:
            self.vo_name = default_vo_name

        # voms port (user)
        if iParams.has_key('voms.port.number'):
            self.voms_port = iParams['voms.port.number']
        else:
            glib.printFailedMessage()
            glib.printErrorMessage("Missing parameter: \"voms.port.number\" not found.")
            return 1            

        #--------------------------------------------------------
        # voms admin specific configuration values
        #--------------------------------------------------------
        
        if self.voms_admin_install == "true":
        
            # smtp host (user)
            if iParams.has_key('voms.admin.smtp.host'):
                self.smtp_host = iParams['voms.admin.smtp.host']
            else:
                glib.printFailedMessage()
                glib.printErrorMessage("Missing parameter: \"voms.admin.smtp.host\" not found.")
                return 1    

            # email address for voms messages (user)
            if iParams.has_key('voms.admin.notification.e-mail'):
                self.voms_mail_address = iParams['voms.admin.notification.e-mail']
            else:
                glib.printFailedMessage()
                glib.printErrorMessage("Missing parameter: \"voms.admin.notification.e-mail\" not found.")
                return 1
            
            # request scheduler (temporary bug fix for bug 13863)
            if iParams.has_key('voms.admin.requestScheduler.disable'):
                if ((iParams['voms.admin.requestScheduler.disable'] == "true") or
                   (iParams['voms.admin.requestScheduler.disable'] == "false")):
                    self.requestScheduler_disable = iParams['voms.admin.requestScheduler.disable']
                else:
                    glib.printFailedMessage()
                    glib.printErrorMessage("Incorrect parameter given: 'voms.admin.requestScheduler.disable' must be true|false.")
                    return 1            
            else:
                glib.printFailedMessage()
                glib.printErrorMessage("Missing parameter: \"voms.admin.requestScheduler.disable\" not found.")
                return 1            

        #--------------------------------------------------------
        # Database specific variables
        #--------------------------------------------------------

        # database name (user)
        if iParams.has_key('voms.db.name'):
            self.db_name = iParams['voms.db.name']
        else:
            glib.printFailedMessage()
            glib.printErrorMessage("Missing parameter: \"voms.db.name\" not found.")
            return 1            

        # database user name (user)
        if iParams.has_key('voms.db.user.name'):
            self.db_user_name = iParams['voms.db.user.name']
        else:
            glib.printFailedMessage()
            glib.printErrorMessage("Missing parameter: \"voms.db.user.name\" not found.")
            return 1    

        # database user password (user)
        if iParams.has_key('voms.db.user.password'):
            self.db_user_password = iParams['voms.db.user.password']
        else:
            glib.printFailedMessage()
            glib.printErrorMessage("Missing parameter: \"voms.db.user.password\" not found.")
            return 1    

        # database host (user)
        if iParams.has_key('voms.db.host'):
            self.db_host = iParams['voms.db.host']
        else:
            glib.printFailedMessage()
            glib.printErrorMessage("Missing parameter: \"voms.db.host\" not found.")
            return 1

        #--------------------------------------------------------
        # Get mysql options (if choosen)
        #--------------------------------------------------------
        if self.db_type== "mysql":

            # mysql admin password => to create users (advanced)
            self.setMysqlAdminPassword = params['set.mysql.root.password']

            # mysql admin password => to create user (user)
            if params.has_key('voms.mysql.admin.password'):
                self.db_admin_password = params['voms.mysql.admin.password']

                # check for non empty password - bug 10441
                if self.db_admin_password == "":
                    glib.printFailedMessage()
                    glib.printErrorMessage("Empty password given for parameter \"voms.mysql.admin.password\".")
                    glib.printErrorMessage("VOMS does not support unprotected DBs.")
                    glib.printErrorMessage("Please set the password for your MySQL database by issuing:")
                    glib.printErrorMessage("  /usr/bin/mysqladmin -u root password 'yourPassword'")
                    glib.printErrorMessage("  /usr/bin/mysqladmin -u root -h %s password 'yourPassword'" % glib.fq_hostname)
                    glib.printErrorMessage("and specify the password in the parameter \"voms.mysql.admin.password\"")
                    glib.printErrorMessage("before rerunning the configuration.")
                    return 1
                    
                elif self.db_host == "localhost":
                    
                    if self.mysql.checkMySQLConfiguration(self.db_admin_password,self.setMysqlAdminPassword):
                        return 1

            else:
                glib.printFailedMessage()
                glib.printErrorMessage("Missing parameter: \"voms.mysql.admin.password\" not found.")
                return 1                

            # mysql admin user => to create users (advanced)
            self.db_admin_name = iParams['voms.mysql.admin.name']
                
            # mysql db libraries (advanced)
            self.db_library = iParams['voms.db.mysql.library']
                                            
            # mysql db port (advanced)
            self.db_port = iParams['voms.db.mysql.port']
  
        #--------------------------------------------------------
        # Get oracle options (if choosen)
        #--------------------------------------------------------        
        else:

            # oracle db libraries (advanced)
            self.db_library = iParams['voms.db.oracle.library']

            # oracle db port (advanced)
            self.db_port = iParams['voms.db.oracle.port']
            
        print "Checking VO specific configuration values                 ",
        glib.printOkMessage() 
                                                            
        #--------------------------------------------------------
        # Configure only voms 
        #--------------------------------------------------------                        
        if self.voms_admin_install == 'false':
  
            print "Configuring voms ...                       "

            if self.verbose:
                errorcode = os.system("%s/libexec/voms/voms_install_db --voms-vo %s --db-type %s --db %s --db-admin %s --db-pwd '%s' --voms-name %s --voms-pwd '%s' --sqlloc %s --port %s" % \
                    (os.environ['GLITE_LOCATION'], self.vo_name, self.db_type, self.db_name, self.db_admin_name, self.db_admin_password, self.db_user_name, self.db_user_password, self.db_library, self.voms_port))
            else:
                status_output = commands.getstatusoutput("%s/libexec/voms/voms_install_db --voms-vo %s --db-type %s --db %s --db-admin %s --db-pwd '%s' --voms-name %s --voms-pwd '%s' --sqlloc %s --port %s" % \
                    (os.environ['GLITE_LOCATION'], self.vo_name, self.db_type, self.db_name, self.db_admin_name, self.db_admin_password, self.db_user_name, self.db_user_password, self.db_library, self.voms_port))                                                          
                errorcode = status_output[0]/256
                output = status_output[1]
            if errorcode:
                if not self.verbose:
                    print output
                print "Configuring voms                                          ",
                glib.printFailedMessage()
                return 1

            print "Configuring voms                                          ",                
            glib.printOkMessage()            
            
        #--------------------------------------------------------
        # Configure voms and voms-admin
        #--------------------------------------------------------                        
        else:
            
            print "Configuring voms and voms-admin (may take a while)        "
            
            if self.db_type== "mysql":
            
                # default command line options
                commandLineOptions = " --vo %s --dbtype %s --dbname %s --dbauser %s --dbapwd '%s' --dbusername %s --dbpassword '%s' --dbhost %s --dbport %s --port %s --smtp-host %s --mail-from %s " % (self.vo_name, 
                         self.db_type, self.db_name, self.db_admin_name, self.db_admin_password, self.db_user_name, self.db_user_password, 
                         self.db_host, self.db_port, self.voms_port, self.smtp_host, self.voms_mail_address)
            
            else:
                
                # default command line options
                commandLineOptions = " --vo %s --dbtype %s --dbname %s --dbusername %s --dbpassword '%s' --dbhost %s --dbport %s --port %s --smtp-host %s --mail-from %s " % (self.vo_name, 
                         self.db_type, self.db_name, self.db_user_name, self.db_user_password, 
                         self.db_host, self.db_port, self.voms_port, self.smtp_host, self.voms_mail_address)
            
            # supplementary command line options if specified
            
            # tomcat group if specified - the same is true for voms group
            if iParams.has_key('tomcat.user.group'):
                commandLineOptions = commandLineOptions + " --tomcat-group %s " % iParams['tomcat.user.group']
            
            # prepared for next version of voms configuration script
            #if iParams.has_key('voms.group'):
            #    commandLineOptions = commandLineOptions + " --voms-group %s " % iParams['voms.user.group']
                
            # admin cert if given
            if iParams.has_key('voms.admin.certificate'):
                if iParams['voms.admin.certificate'] != "":
                    commandLineOptions = commandLineOptions + " --admincert %s " % iParams['voms.admin.certificate']
            
            # verbose mode if enabled
            if self.verbose:
                commandLineOptions = commandLineOptions + " --verbose "
            
            
            if self.verbose:            
                errorcode = os.system("%s/sbin/voms-admin-configure install %s" % (os.environ['GLITE_LOCATION'], commandLineOptions))                 
            else:
                status_output = commands.getstatusoutput("%s/sbin/voms-admin-configure install %s" % (os.environ['GLITE_LOCATION'], commandLineOptions))
                errorcode = status_output[0]/256
                output = status_output[1]
            if errorcode:
                if not self.verbose:
                    print output
                print "Configuring voms and voms-admin                           ",
                glib.printFailedMessage()
                return 1
            
            # disable request scheduler if option is set (temporary bug fix for bug 13863)
            if (self.requestScheduler_disable == "true"):
                
                # update entries in $GLITE_LOCATION_VAR/etc/voms-admin/VONAME
                errorcodeAll = 0
                errorcodeAll += self.setFileKeyValuePair("%s/etc/voms-admin/%s/voms.service.properties" % (os.environ['GLITE_LOCATION_VAR'], self.vo_name), "voms.complete.requests.expire.after", "0")
                errorcodeAll += self.setFileKeyValuePair("%s/etc/voms-admin/%s/voms.service.properties" % (os.environ['GLITE_LOCATION_VAR'], self.vo_name), "voms.incomplete.requests.time.out.after", "0")
                errorcodeAll += self.setFileKeyValuePair("%s/etc/voms-admin/%s/voms.service.properties" % (os.environ['GLITE_LOCATION_VAR'], self.vo_name), "voms.request.expire.task.period.hours", "0")
                errorcodeAll += self.setFileKeyValuePair("%s/etc/voms-admin/%s/voms.service.properties" % (os.environ['GLITE_LOCATION_VAR'], self.vo_name), "voms.request.timeout.task.period.hours", "0")
                errorcodeAll += self.setFileKeyValuePair("%s/etc/voms-admin/%s/voms.service.properties" % (os.environ['GLITE_LOCATION_VAR'], self.vo_name), "voms.request.purge.task.period.seconds", "0")
                if errorcodeAll != 0:
                    print "Configuring voms and voms-admin                           ",                
                    glib.printFailedMessage()
                    return 1
            else:
                # update entries in $GLITE_LOCATION_VAR/etc/voms-admin/VONAME
                errorcodeAll = 0
                errorcodeAll += self.setFileKeyValuePair("%s/etc/voms-admin/%s/voms.service.properties" % (os.environ['GLITE_LOCATION_VAR'], self.vo_name), "voms.complete.requests.expire.after", "60")
                errorcodeAll += self.setFileKeyValuePair("%s/etc/voms-admin/%s/voms.service.properties" % (os.environ['GLITE_LOCATION_VAR'], self.vo_name), "voms.incomplete.requests.time.out.after", "30")
                errorcodeAll += self.setFileKeyValuePair("%s/etc/voms-admin/%s/voms.service.properties" % (os.environ['GLITE_LOCATION_VAR'], self.vo_name), "voms.request.expire.task.period.hours", "24")
                errorcodeAll += self.setFileKeyValuePair("%s/etc/voms-admin/%s/voms.service.properties" % (os.environ['GLITE_LOCATION_VAR'], self.vo_name), "voms.request.timeout.task.period.hours", "24")
                errorcodeAll += self.setFileKeyValuePair("%s/etc/voms-admin/%s/voms.service.properties" % (os.environ['GLITE_LOCATION_VAR'], self.vo_name), "voms.request.purge.task.period.seconds", "1800")
                if errorcodeAll != 0:
                    print "Configuring voms and voms-admin                           ",                
                    glib.printFailedMessage()
                    return 1
                
            print "Configuring voms and voms-admin                           ",                
            glib.printOkMessage()

        #--------------------------------------------------------
        # Configure R-GMA servicetool
        #--------------------------------------------------------                        

        if self.rgma_servicetool_activate=="true":
        
            # Create one instance
            servicetoolInstance = gliteRgmaServicetoolInstance(glib, "%s_%s_%s" % (os.environ['HOSTNAME'], self.vo_name, self.serviceType))
        
            # set servicetool instance params
            servicetoolInstance.setServiceType(self.serviceType)
            servicetoolInstance.setServiceVersion(self.version)
            servicetoolInstance.setStatusScript("%s --endpoint %s%s/ --key %s --certificate %s" % 
                (self.serviceStatusScript, self.serviceVoEndpoint, self.vo_name, self.rgmaKeyPath, self.rgmaCertificatePath))
            servicetoolInstance.setEndpoint("%s%s" % (self.serviceEndpointBase, self.vo_name))
            servicetoolInstance.setServiceName("%s%s" % (self.serviceName, self.vo_name))
            servicetoolInstance.setWsdlUrl("%s%s%s" % (self.serviceWsdlUrlBase1, self.vo_name, self.serviceWsdlUrlBase2))
            servicetoolInstance.add()
                           
        print "Configuring VO '%s'            " % default_vo_name,
        glib.printOkMessage()

        return 0
 
 
    #################################################################
    # setFileKeyValuePair
    #
    # set a key value pair in a file
    #
    # return 0 ok
    #        1 error
    #
    #################################################################     
    def setFileKeyValuePair(self, fileName, key, value):
        
        try:                                        
            input = open(fileName, 'r')
            output = open(fileName + ".tmp", 'w')
            
            inlines = input.readlines()

            found = 0
            
            # exchange DB name, user and password and retention period
            for line in inlines:
                if line.startswith(key):
                    output.write("%s %s\n" % (key, value))
                    found = 1
                else:
                    output.write(line)
            
            if found == 0:
                output.write("%s %s\n" % (key, value))
                    
            # close
            output.close()        
            input.close()
                
            # rename file
            os.rename(fileName + ".tmp", fileName)
            
            return 0
        
        except IOError, ex:
            print "Error processing file '%s'." % fileName
            return 1   
         
        
    #################################################################
    # start
    #
    # start voms server
    #
    # return 0 ok
    #        1 error
    #
    #################################################################
    def start(self):
        
        print "\nxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
        print "          Starting the %s" % self.friendly_name
        print "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx\n"

        #--------------------------------------------------------
        # Get and check configuration parameters what to start
        #--------------------------------------------------------

        # voms only or voms + voms-admin (user)
        if params.has_key('voms.admin.install'):
            self.voms_admin_install = params['voms.admin.install']
            if (self.voms_admin_install != "true") and (self.voms_admin_install != "false"):
                glib.printErrorMessage("Incorrect parameter 'voms.admin.install' given.")
                glib.printErrorMessage("Must be 'true|false'. (Given parameter is '%s')" % self.voms_admin_install)
                return 1            
        else:
            glib.printFailedMessage()
            glib.printErrorMessage("Missing parameter: \"voms.admin.install\" not found.")
            return 1                  

        # activate rgma servicetool (advanced)
        self.rgma_servicetool_activate = params['rgma.servicetool.activate']
        if (self.rgma_servicetool_activate != "true") and (self.rgma_servicetool_activate != "false"):
            glib.printFailedMessage()
            glib.printErrorMessage("Incorrect parameter 'rgma.servicetool.activate' given.")
            glib.printErrorMessage("Must be 'true|false'. (Given parameter is '%s')" % self.rgma_servicetool_activate)
            return 1            
        
        #--------------------------------------------------------
        # Start VOMS
        #--------------------------------------------------------        
        os.system("%s/etc/init.d/voms start" % self.params['GLITE_LOCATION'])

        #--------------------------------------------------------
        # Start VOMS admin
        #--------------------------------------------------------        
        if self.voms_admin_install == "true":

            # Start Tomcat
            if self.tomcat.start() != 0:
                return 1
        
            # Start VOMS admin
            os.system("%s/etc/init.d/voms-admin start" % self.params['GLITE_LOCATION'])

        #--------------------------------------------------------
        # Start Servicetool
        #--------------------------------------------------------        
        print
        if self.rgma_servicetool_activate=="true":
            if rgmaServicetool.start():
                return 1
        
        return 0


    #################################################################
    # stop
    #
    # stop voms server
    #
    # return 0 ok
    #        1 error
    #
    #################################################################        
    def stop(self):

        print "\nxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
        print "          Stopping the %s" % self.friendly_name
        print "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx\n"
 
        #--------------------------------------------------------
        # Get and check configuration parameters what to stop
        #--------------------------------------------------------
        
        # voms only or voms + voms-admin (user)
        if params.has_key('voms.admin.install'):
            self.voms_admin_install = params['voms.admin.install']
            if (self.voms_admin_install != "true") and (self.voms_admin_install != "false"):
                glib.printErrorMessage("Incorrect parameter 'voms.admin.install' given.")
                glib.printErrorMessage("Must be 'true|false'. (Given parameter is '%s')" % self.voms_admin_install)
                return 1            
        else:
            glib.printFailedMessage()
            glib.printErrorMessage("Missing parameter: \"voms.admin.install\" not found.")
            return 1                  
        
        # activate rgma servicetool (advanced)
        self.rgma_servicetool_activate = params['rgma.servicetool.activate']
        if (self.rgma_servicetool_activate != "true") and (self.rgma_servicetool_activate != "false"):
            glib.printFailedMessage()
            glib.printErrorMessage("Incorrect parameter 'rgma.servicetool.activate' given.")
            glib.printErrorMessage("Must be 'true|false'. (Given parameter is '%s')" % self.rgma_servicetool_activate)
            return 1            
        
        #--------------------------------------------------------        
        # Stop VOMS admin
        #--------------------------------------------------------        
        if self.voms_admin_install == 'true':

            # stop voms-admin
            os.system("%s/etc/init.d/voms-admin stop" % self.params['GLITE_LOCATION'])

            # Stop tomcat
            self.tomcat.stop()
        
        #--------------------------------------------------------        
        # Stop VOMS
        #--------------------------------------------------------        
        os.system("%s/etc/init.d/voms stop" % self.params['GLITE_LOCATION'])

        #--------------------------------------------------------        
        # Stop the servicetool
        #--------------------------------------------------------                
        if self.rgma_servicetool_activate=="true":
            if rgmaServicetool.stop():
                return 1        
        print
        
        return 0
        
        
    #################################################################
    # status
    #
    # print status of VOMS server
    #
    # return 0 ok
    #        otherwise: error
    #
    #################################################################        
    def status(self):

        print "\nxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
        print "          Checking status of %s" % self.friendly_name
        print "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx\n"

        #--------------------------------------------------------
        # Get and check configuration parameters what to show status
        #--------------------------------------------------------

        # install voms only or voms + voms-admin (user)
        if params.has_key('voms.admin.install'):
            self.voms_admin_install = params['voms.admin.install']
            if (self.voms_admin_install != "true") and (self.voms_admin_install != "false"):
                glib.printErrorMessage("Incorrect parameter 'voms.admin.install' given.")
                glib.printErrorMessage("Must be 'true|false'. (Given parameter is '%s')" % self.voms_admin_install)
                return 1            
        else:
            glib.printFailedMessage()
            glib.printErrorMessage("Missing parameter: \"voms.admin.install\" not found.")
            return 1                  

        # activate rgma servicetool (advanced)
        self.rgma_servicetool_activate = params['rgma.servicetool.activate']
        if (self.rgma_servicetool_activate != "true") and (self.rgma_servicetool_activate != "false"):
            glib.printFailedMessage()
            glib.printErrorMessage("Incorrect parameter 'rgma.servicetool.activate' given.")
            glib.printErrorMessage("Must be 'true|false'. (Given parameter is '%s')" % self.rgma_servicetool_activate)
            return 1            
        
        error_level = 0

        #--------------------------------------------------------
        # VOMS admin
        #--------------------------------------------------------
        if self.voms_admin_install == 'true':
            print "\n----------------------------------------------------"
            print " Status of VOs VOMS-admin server:"
            print "----------------------------------------------------"
            retval = os.system("%s/etc/init.d/voms-admin status" % self.params['GLITE_LOCATION'])
            if retval != 0:
                error_level = retval

        #--------------------------------------------------------
        # VOMS 
        #--------------------------------------------------------            
        print "\n----------------------------------------------------"
        print " Status of VOs VOMS server:"
        print "----------------------------------------------------"
        retval = os.system("%s/etc/init.d/voms status" % self.params['GLITE_LOCATION'])
        if retval != 0:
            error_level = retval
        
        # ------------------------------------------------------------
        # R-GMA service publisher
        # ------------------------------------------------------------
        if self.rgma_servicetool_activate=="true":
            if rgmaServicetool.status():
                error_level = error_level + 1
                    
        return error_level
     

    #################################################################
    # start one vo
    #
    # start single vo
    #
    # return 0 ok
    #        1 error
    #
    #################################################################
    def startVo(self,voName):
        
        print "\nxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
        print "          Starting the VO '%s' %s" % (voName, self.friendly_name)
        print "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx\n"

        #--------------------------------------------------------
        # Get and check configuration parameters what to start
        #--------------------------------------------------------

        # voms only or voms + voms-admin (user)
        if params.has_key('voms.admin.install'):
            self.voms_admin_install = params['voms.admin.install']
            if (self.voms_admin_install != "true") and (self.voms_admin_install != "false"):
                glib.printErrorMessage("Incorrect parameter 'voms.admin.install' given.")
                glib.printErrorMessage("Must be 'true|false'. (Given parameter is '%s')" % self.voms_admin_install)
                return 1            
        else:
            glib.printFailedMessage()
            glib.printErrorMessage("Missing parameter: \"voms.admin.install\" not found.")
            return 1                  

        # activate rgma servicetool (advanced)
        self.rgma_servicetool_activate = params['rgma.servicetool.activate']
        if (self.rgma_servicetool_activate != "true") and (self.rgma_servicetool_activate != "false"):
            glib.printFailedMessage()
            glib.printErrorMessage("Incorrect parameter 'rgma.servicetool.activate' given.")
            glib.printErrorMessage("Must be 'true|false'. (Given parameter is '%s')" % self.rgma_servicetool_activate)
            return 1            
        
        #--------------------------------------------------------
        # Start VOMS for specified VO
        #--------------------------------------------------------        
        retval = os.system("%s/etc/init.d/voms start %s" % (self.params['GLITE_LOCATION'], voName))
        if retval != 0:
            return 1

        #--------------------------------------------------------
        # Start VOMS admin for specified VO and tomcat if necessary
        #--------------------------------------------------------        
        if self.voms_admin_install == "true":

            # Check tomcat status and start if necessary
            if self.tomcat.status() != 0:
                if self.tomcat.start() !=0:
                    return 1
        
            # Start VOMS admin
            retval = os.system("%s/etc/init.d/voms-admin start %s" % (self.params['GLITE_LOCATION'],voName))
            if retval != 0:
                return 1

        #--------------------------------------------------------
        # Start RGMA Servicetool if necessary
        #--------------------------------------------------------        
        print
        if self.rgma_servicetool_activate=="true":
            if rgmaServicetool.start():
                return 1
        
        return 0
    
           
        
    #################################################################
    # stop one vo
    #
    # stop single vo -leaves alone tomcat and RGMA servicetool
    #
    # return 0 ok
    #        1 error
    #
    #################################################################
    def stopVo(self,voName):       
        
        print "\nxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
        print "          Stopping the VO '%s' %s" % (voName, self.friendly_name)
        print "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx\n"

        #--------------------------------------------------------
        # Get and check configuration parameters what to stop
        #--------------------------------------------------------

        # voms only or voms + voms-admin (user)
        if params.has_key('voms.admin.install'):
            self.voms_admin_install = params['voms.admin.install']
            if (self.voms_admin_install != "true") and (self.voms_admin_install != "false"):
                glib.printErrorMessage("Incorrect parameter 'voms.admin.install' given.")
                glib.printErrorMessage("Must be 'true|false'. (Given parameter is '%s')" % self.voms_admin_install)
                return 1            
        else:
            glib.printFailedMessage()
            glib.printErrorMessage("Missing parameter: \"voms.admin.install\" not found.")
            return 1                  

        # activate rgma servicetool (advanced)
        self.rgma_servicetool_activate = params['rgma.servicetool.activate']
        if (self.rgma_servicetool_activate != "true") and (self.rgma_servicetool_activate != "false"):
            glib.printFailedMessage()
            glib.printErrorMessage("Incorrect parameter 'rgma.servicetool.activate' given.")
            glib.printErrorMessage("Must be 'true|false'. (Given parameter is '%s')" % self.rgma_servicetool_activate)
            return 1            
        
        #--------------------------------------------------------
        # Stop VOMS for specified VO
        #--------------------------------------------------------        
        print "Stopping voms for VO %s ..." % voName
        retval = os.system("%s/etc/init.d/voms stop %s" % (self.params['GLITE_LOCATION'], voName))
        if retval == 0:
            print "Stopping voms for VO %s                  " % voName, 
            glib.printOkMessage()                            
        else:
            print "Stopping voms for VO %s                  " % voName, 
            glib.printFailedMessage()
            return 1                        

        #--------------------------------------------------------
        # Start VOMS admin for specified VO and tomcat
        #--------------------------------------------------------        
        if self.voms_admin_install == "true":

            #--------------------------------------------------------                    
            # We leave tomcat alone
            #--------------------------------------------------------                    
        
            # Stop VOMS admin
            print "Stopping voms-admin for VO %s ..." % voName
            retval = os.system("%s/etc/init.d/voms-admin stop %s" % (self.params['GLITE_LOCATION'],voName))
            if retval == 0:
                print "Stopping voms-admin for VO %s                  " % voName, 
                glib.printOkMessage()                            
            else:
                print "Stopping voms-admin for VO %s                  " % voName, 
                glib.printFailedMessage()
                return 1                        

        #--------------------------------------------------------                
        # we leave RGMA servicetool alone
        #--------------------------------------------------------                
        
        return 0


    #################################################################
    # statusVo
    #
    # print status of VOMS server for a VO
    #
    # return 0 running
    #        otherwise: not running / error
    #
    #################################################################        
    def statusVo(self, voName):

        print "\nxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
        print "          Checking status of VO '%s' of %s" % (voName, self.friendly_name)
        print "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx\n"

        #--------------------------------------------------------
        # Get and check configuration parameters what to show status
        #--------------------------------------------------------

        # install voms only or voms + voms-admin (user)
        if params.has_key('voms.admin.install'):
            self.voms_admin_install = params['voms.admin.install']
            if (self.voms_admin_install != "true") and (self.voms_admin_install != "false"):
                glib.printErrorMessage("Incorrect parameter 'voms.admin.install' given.")
                glib.printErrorMessage("Must be 'true|false'. (Given parameter is '%s')" % self.voms_admin_install)
                return 1            
        else:
            glib.printFailedMessage()
            glib.printErrorMessage("Missing parameter: \"voms.admin.install\" not found.")
            return 1                  

        # activate rgma servicetool (advanced)
        self.rgma_servicetool_activate = params['rgma.servicetool.activate']
        if (self.rgma_servicetool_activate != "true") and (self.rgma_servicetool_activate != "false"):
            glib.printFailedMessage()
            glib.printErrorMessage("Incorrect parameter 'rgma.servicetool.activate' given.")
            glib.printErrorMessage("Must be 'true|false'. (Given parameter is '%s')" % self.rgma_servicetool_activate)
            return 1            
        
        error_level = 0

        #--------------------------------------------------------
        # VOMS admin for given VO
        #--------------------------------------------------------
        if self.voms_admin_install == 'true':
            retval = os.system("%s/etc/init.d/voms-admin status %s" % (self.params['GLITE_LOCATION'], voName))
            if retval != 0:
                error_level = retval

        #--------------------------------------------------------
        # VOMS for given VO
        #--------------------------------------------------------            
        retval = os.system("%s/etc/init.d/voms status %s " % (self.params['GLITE_LOCATION'], voName))
        if retval != 0:
            error_level = retval
                    
        return error_level
    
    
    ####################################
    # removeVo
    #       
    # Method to remove VO from VOMS. VO is stooped and removed
    #
    # params voName name of vo to be removed
    #
    # return 0 ok 
    #        1 error
    #
    ####################################
    def removeVo(self, voName, removedb=0):
 
        print "\nxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
        print "            Removing vo %s from %s" % (voName, self.friendly_name)
        print "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx\n"
        
        #--------------------------------------------------------        
        # Check if voms or voms-admin is used
        #--------------------------------------------------------        

        # voms only or voms + voms-admin (user)
        if params.has_key('voms.admin.install'):
            self.voms_admin_install = params['voms.admin.install']
            if (self.voms_admin_install != "true") and (self.voms_admin_install != "false"):
                glib.printErrorMessage("Incorrect parameter 'voms.admin.install' given.")
                glib.printErrorMessage("Must be 'true|false'. (Given parameter is '%s')" % self.voms_admin_install)
                return 1            
        else:
            glib.printFailedMessage()
            glib.printErrorMessage("Missing parameter: \"voms.admin.install\" not found.")
            return 1                  

        # if voms: function not supported
        if self.voms_admin_install == 'false':
            glib.printErrorMessage("Option only supported for voms-admin:")
            glib.printErrorMessage("Parameter \"voms.admin.install\" must be set to \"true\".")
            return 1
 
        #--------------------------------------------------------
        # Get and check common configuration parameters
        #--------------------------------------------------------

        # Decide on database type: MySQL or ORACLE (user)
        if params.has_key('voms.db.type'):        

            if params['voms.db.type'] == 'mysql':
                                
                # db type
                self.db_type = "mysql"
                                
            elif params['voms.db.type'] == 'oracle':
                    
                # db type
                self.db_type = "oracle"
                                
            else:
            
                # Incorrect parameter given
                glib.printFailedMessage()
                glib.printErrorMessage("Incorrect parameter 'voms.db.type' given.")
                glib.printErrorMessage("Must be 'mysql|oracle'. (Given parameter is '%s')" % params['voms.db.type'])
                return 1
        else:
            glib.printFailedMessage()
            glib.printErrorMessage("Parameter \"voms.db.type\" not found.")
            return 1

        #--------------------------------------------------------
        # stop VO
        #--------------------------------------------------------            
        self.stopVo(voName)
                
        #--------------------------------------------------------
        # remove VO
        #--------------------------------------------------------            

        # default command line options
        if removedb:
            commandLineOptions = " --vo %s --undeploy-database" % (voName)
        else:
            commandLineOptions = " --vo %s" % (voName)

        if self.verbose:
            errorcode = os.system("%s/sbin/voms-admin-configure remove %s" % (os.environ['GLITE_LOCATION'], commandLineOptions))                 
        else:
            status_output = commands.getstatusoutput("%s/sbin/voms-admin-configure remove %s" % (os.environ['GLITE_LOCATION'], commandLineOptions))
            errorcode = status_output[0]/256
            output = status_output[1]                
        if errorcode:
            if not self.verbose:
                print output
            return 1

        #--------------------------------------------------------
        # Remove VO entry from RGMA servicetool
        #--------------------------------------------------------
        if rgmaServicetool.removeServiceFile("%s_%s_%s" % (os.environ['HOSTNAME'], voName, self.serviceType)) != 0:
            return 1
 
        print "Removing VO '%s'                                " % voName,
        glib.printOkMessage()

        return 0


    ####################################
    # configureVo
    #       
    # Method to configue a VO for VOMS
    # VO is stopped if existing and running
    #
    # params voName name of vo to be added
    #
    # return 0 ok 
    #        1 error
    #
    ####################################
    def configureVo(self, voName):
        
        print "\nxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
        print "            Configuring vo %s for %s" % (voName, self.friendly_name)
        print "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx\n"
        
        #--------------------------------------------------------        
        # Get common configuration parameters
        #--------------------------------------------------------        

        print "Checking the common configuration values                  ",                

        # Decide on database type: MySQL or ORACLE (user)   
        if params.has_key('voms.db.type'):        
            if params['voms.db.type'] == 'mysql':
                self.db_type = "mysql"
            elif params['voms.db.type'] == 'oracle':
                self.db_type = "oracle"            
            else:
                # Incorrect parameter given
                glib.printFailedMessage()
                glib.printErrorMessage("Incorrect parameter 'voms.db.type' given.")
                glib.printErrorMessage("Must be 'mysql|oracle'. (Given parameter is '%s')" % params['voms.db.type'])
                return 1
        else:
            glib.printFailedMessage()
            glib.printErrorMessage("Parameter \"voms.db.type\" not found for.")
            return 1

        # Decide on: install voms only or voms + voms-admin (user)
        if params.has_key('voms.admin.install'):
            self.voms_admin_install = params['voms.admin.install']
            if (self.voms_admin_install != "true") and (self.voms_admin_install != "false"):
                glib.printFailedMessage()
                glib.printErrorMessage("Incorrect parameter 'voms.admin.install' given.")
                glib.printErrorMessage("Must be 'true|false'. (Given parameter is '%s')" % self.voms_admin_install)
                return 1            
        else:
            glib.printFailedMessage()
            glib.printErrorMessage("Missing parameter: \"voms.admin.install\" not found.")
            return 1        
        
        # activate rgma servicetool (advanced)
        self.rgma_servicetool_activate = params['rgma.servicetool.activate']
        if (self.rgma_servicetool_activate != "true") and (self.rgma_servicetool_activate != "false"):
            glib.printFailedMessage()
            glib.printErrorMessage("Incorrect parameter 'rgma.servicetool.activate' given.")
            glib.printErrorMessage("Must be 'true|false'. (Given parameter is '%s')" % self.rgma_servicetool_activate)
            return 1            

        # MySQL configuration params
        if self.db_type == "mysql":

            # fix for bug no
            # max connections for mysql (advanced)
            self.mysql_maxConnections = params['voms.db.mysql.maxConnections']
            
        # Oracle configuration params
        if self.db_type == "oracle":
            
            # location for oracle instants client (advanced)
            self.oracle_instantclient_location = params['voms.db.oracle.instantclient.location']        

        glib.printOkMessage()
        
        #--------------------------------------------------------
        # Get R-GMA settings
        #--------------------------------------------------------
        if self.rgma_servicetool_activate=="true":
     
            # rgma user name (system)
            rgmaUserName = params['rgma.user.name']

            # certificate path (system)
            userCertificatePath = params['user.certificate.path']

            # derive rgma user key and certificate path
            rgmaUserHomeDir     = pwd.getpwnam(rgmaUserName)[5] 
            self.rgmaCertificatePath = "%s/%s/hostcert.pem" % (rgmaUserHomeDir, userCertificatePath)
            self.rgmaKeyPath         = "%s/%s/hostkey.pem"  % (rgmaUserHomeDir, userCertificatePath)
        
        #--------------------------------------------------------        
        # Get list of voms vo instances
        #--------------------------------------------------------        
        instances = self.glib.getInstances('voms')

        if len(instances) > 0:
            
            for instance in instances:
                
                #--------------------------------------------------------
                # Extract vo name for instance
                #--------------------------------------------------------
                
                # extract vo name from 'default:voName' string
                vo_name_array = instance.split(":")
                default_vo_name = vo_name_array[1]
                 
                try:
                    iParams = self.glib.getInstanceParams(instance)
                except ValueError, msg:
                    print "Checking VO specific configuration values                 ",
                    glib.printFailedMessage()
                    glib.printErrorMessage("%s, Cannot continue" % msg)
                    return 1
                except LookupError, msg:
                    print "Checking VO specific configuration values                 ",            
                    glib.printFailedMessage()
                    glib.printErrorMessage(msg)
                    return 1

                # vo name - if not present: assign name of instance (user)
                if iParams.has_key('vo.name'):
                    self.vo_name = iParams['vo.name']
                else:
                    self.vo_name = default_vo_name
            
                #--------------------------------------------------------
                # Check if it is the vo we are looking for
                #--------------------------------------------------------
                if self.vo_name == voName:
            
                    #--------------------------------------------------------
                    # stop VO
                    #--------------------------------------------------------            
                    self.stopVo(voName)
                        
                    #--------------------------------------------------------
                    # configure VO
                    #--------------------------------------------------------            
                    if self.configureVomsInstance(instance):
                        return 1
                    
                    #--------------------------------------------------------
                    # RGMA servicetool: configure servicetool
                    #--------------------------------------------------------        
                    print
                    if self.rgma_servicetool_activate=="true":
                        if rgmaServicetool.configure(self.glib):
                            return 1
                            
                    return 0
                                        
        glib.printWarningMessage("Specified VO not available in configuration.")
        return 1


    #################################################################
    # showConfiguration
    #
    # print major configuration
    #
    # return 0 ok
    #        1 error
    #
    #################################################################
    def showConfigurationForOneVo(self, voName):
        
        print "  VOMS VO settings"
        print "     --------------------------------------------------------------"

        # loop over VO's
        instances = self.glib.getInstances('voms')

        if len(instances) > 0:
            
            for instance in instances:
                
                #--------------------------------------------------------
                # Extract vo name for instance
                #--------------------------------------------------------
                
                # extract vo name from 'default:voName' string
                vo_name_array = instance.split(":")
                default_vo_name = vo_name_array[1]
                 
                try:
                    iParams = self.glib.getInstanceParams(instance)
                except ValueError, msg:
                    print "Checking VO specific configuration values                 ",
                    glib.printFailedMessage()
                    glib.printErrorMessage("%s, Cannot continue" % msg)
                    return 1
                except LookupError, msg:
                    print "Checking VO specific configuration values                 ",            
                    glib.printFailedMessage()
                    glib.printErrorMessage(msg)
                    return 1

                # vo name - if not present: assign name of instance (user)
                if iParams.has_key('vo.name'):
                    self.vo_name = iParams['vo.name']
                else:
                    self.vo_name = default_vo_name
            
                #--------------------------------------------------------
                # Check if it is the vo we are looking for
                #--------------------------------------------------------
                if self.vo_name == voName:
                    
                    if self.showVoConfiguration(instance):
                        return 1
                    
                    return 0
        
        return 1        
        
                
    #################################################################
    # showConfiguration
    #
    # print major configuration
    #
    # return 0 ok
    #        1 error
    #
    #################################################################
    def showConfiguration(self):
        
        self.copyright()
        self.showVersion()
        glib.print_params(params)

        print "\nxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
        print "             Main settings of %s" % self.friendly_name
        print "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx\n"

        print "  VOMS SERVER"
        print "     --------------------------------------------------------------"
        if params.has_key('voms.db.type'):
            print "     \033[34m[DB type]\x1b[0m            %s " % params['voms.db.type']
        else:
            glib.printErrorMessage("Parameter 'voms.db.type' not found.")

        vomsAdminInstall = 0
        if params.has_key('voms.admin.install'):
            print "     \033[34m[VOMS-admin enabled]\x1b[0m %s " % params['voms.admin.install']
            vomsAdminInstall = 1
        else:
            glib.printErrorMessage("Parameter 'voms.admin.install' not found.")            
        if vomsAdminInstall == 1:
            print "     \033[34m[VOMS endpoint]\x1b[0m      https://%s:8443/vomses/" % os.environ['HOSTNAME']
        print "     --------------------------------------------------------------"
        print
        
        
        print "  VOMS VO settings"
        print "     --------------------------------------------------------------"

        # loop over VO's
        instances = self.glib.getInstances('voms')

        if len(instances) > 0:
            for instance in instances:
                if self.showVoConfiguration(instance):
                    return

        print
        rgmaServicetool.showServices()
        
        return 0
    
              
    #################################################################
    # showVoConfiguration
    #
    # print major configuration for one VO
    #
    # return 0 ok
    #        1 error
    #
    #################################################################
    def showVoConfiguration(self, name):

        # extract vo name from 'default:voName' string
        vo_name_array = name.split(":")
        default_vo_name = vo_name_array[1]
         
        try:
            iParams = self.glib.getInstanceParams(name)
        except ValueError, msg:
            glib.printFailedMessage()
            glib.printErrorMessage("%s, Cannot continue" % msg)
            return 1
        except LookupError, msg:
            glib.printFailedMessage()
            glib.printErrorMessage(msg)
            return 1

        #--------------------------------------------------------
        # voms configuration values
        #--------------------------------------------------------

        # vo name - if not present: assign name of instance
        if iParams.has_key('vo.name'):
            vo_name = iParams['vo.name']
        else:
            vo_name = default_vo_name

        # voms-admin install
        if params.has_key('voms.admin.install'):
            voms_admin_install = params['voms.admin.install']        

        # Decide on database type: MySQL or ORACLE        
        if params.has_key('voms.db.type'):        

            if params['voms.db.type'] == 'mysql':
                                
                # db type
                db_type = "mysql"
                                
            elif params['voms.db.type'] == 'oracle':
                    
                # db type
                db_type = "oracle"        

        print
        print "     \033[34m[VO]\x1b[0m %s " % vo_name
        print "         =============================================================="

        # voms port
        if iParams.has_key('voms.port.number'):
            print "         \033[34m[VOMS Port]\x1b[0m        %s " % iParams['voms.port.number']
        else:
            glib.printErrorMessage("Parameter 'voms.port.number' not found.")
        
        #--------------------------------------------------------
        # voms admin specific configuration values
        #--------------------------------------------------------
        
        if voms_admin_install == "true":
        
            # voms port
            if iParams.has_key('voms.admin.smtp.host'):
                print "         \033[34m[VOMS-admin SMTP]\x1b[0m  %s " % iParams['voms.admin.smtp.host']
            else:
                glib.printErrorMessage("Parameter 'voms.admin.smtp.host' not found.")

            # email address for voms messages
            if iParams.has_key('voms.admin.notification.e-mail'):
                print "         \033[34m[VOMS-admin mail]\x1b[0m  %s " % iParams['voms.admin.notification.e-mail']
            else:
                glib.printErrorMessage("Parameter 'voms.admin.notification.e-mail' not found.")

            # database name
            if iParams.has_key('voms.db.name'):
                print "         \033[34m[DB Name]\x1b[0m          %s " % iParams['voms.db.name']
            else:
                glib.printErrorMessage("Parameter 'voms.db.name' not found.")

            # database user name
            if iParams.has_key('voms.db.user.name'):
                print "         \033[34m[DB UserName]\x1b[0m      %s " % iParams['voms.db.user.name']
            else:
                glib.printErrorMessage("Parameter 'voms.db.user.name' not found.")

            # database user password
            if iParams.has_key('voms.db.user.password'):
                print "         \033[34m[DB UserPassword]\x1b[0m  %s " % iParams['voms.db.user.password']
            else:
                glib.printErrorMessage("Parameter 'voms.db.user.password' not found.")

            # database host
            if iParams.has_key('voms.db.host'):
                print "         \033[34m[DB Host]\x1b[0m          %s " % iParams['voms.db.host']
            else:
                glib.printErrorMessage("Parameter 'voms.db.host' not found.")

            if iParams.has_key('voms.admin.certificate'):
                print "         \033[34m[VOMS-admin cert]\x1b[0m  %s " % iParams['voms.admin.certificate']

            if iParams.has_key('voms.admin.requestScheduler.disable'):
                print "         \033[34m[VOMS-admin disable requestScheduler]\x1b[0m %s " % iParams['voms.admin.requestScheduler.disable']

            print "         \033[34m[VOMS VO endpoint]\x1b[0m https://%s:8443/voms/%s" % (os.environ['HOSTNAME'], vo_name)
  
        #--------------------------------------------------------
        # Get mysql options (if choosen)
        #--------------------------------------------------------
        if db_type == "mysql":

            # mysql db admin password
            if iParams.has_key('voms.mysql.admin.name'):
                print "         \033[34m[DB AdminName]\x1b[0m     %s " % iParams['voms.mysql.admin.name']
            else:
                glib.printErrorMessage("Parameter 'voms.mysql.admin.name' not found.")

            # mysql db admin password
            if iParams.has_key('voms.mysql.admin.password'):
                print "         \033[34m[DB AdminPassword]\x1b[0m %s " % iParams['voms.mysql.admin.password']
            else:
                glib.printErrorMessage("Parameter 'voms.mysql.admin.password' not found.")

            # mysql db port
            if iParams.has_key('voms.db.mysql.port'):
                print "         \033[34m[DB Port]\x1b[0m          %s " % iParams['voms.db.mysql.port']
            else:
                glib.printErrorMessage("Parameter 'voms.db.mysql.port' not found.")
              
        #--------------------------------------------------------
        # Get oracle options (if choosen)
        #--------------------------------------------------------        
        elif db_type == "oracle":

            # oracle db port
            if iParams.has_key('voms.db.oracle.port'):
                print "         \033[34m[DB Port]\x1b[0m          %s " % iParams['voms.db.oracle.port']
            else:
                glib.printErrorMessage("Parameter 'voms.db.oracle.port' not found.")                      
        
        print "         =============================================================="
        

#################################################################
# loadDefaultParameters
#
# load default parameters
#
# params params list of parameters
#
# return 0 ok
#        1 error
#
#################################################################
def loadDefaultParameters(params):
                    
    # voms default parameters
    params['glite.installer.verbose'] = "true"
    params['glite.installer.checkcerts'] = "true"
    #params['voms.user.group'] = "voms"
    params['voms.admin.install'] = "true"
    # temporary fix for bug 13863: disable request scheduler
    params['voms.admin.requestScheduler.disable'] = "false"
                    
    # mysql
    params['voms.db.mysql.library']   = "${GLITE_LOCATION}/lib/libvomsmysql.so"   
    params['voms.mysql.admin.name']   = "root"        
    params['voms.db.mysql.port']      = '3306'
    params['set.mysql.root.password'] = 'false'
    params['voms.db.mysql.maxConnections'] = "500"
                
    # oracle
    params['voms.db.oracle.library'] = "${GLITE_LOCATION}/lib/libvomsoracle.so"
    params['voms.db.oracle.port']    = '1521'
    params['voms.db.oracle.instantclient.location'] = "/usr/lib/oracle/10.2.0.1/client/"
    params['voms.db.min.connections'] = "1"
    params['voms.db.max.connections'] = "20"
    params['voms.db.startup.connections'] = "10"
    
    # R-GMA servicetool parameters     
    params['rgma.servicetool.activate'] = "true"
    params['rgma.user.name']            = "rgma"
    params['rgma.user.group']           = "rgma"

    
#################################################################
# set_env
#
# setup environment
# 
#################################################################
def set_env():
    
    # gLite
    glib.export('GLITE_LOCATION');
    glib.export('GLITE_LOCATION_VAR');
    glib.export('GLITE_LOCATION_LOG');
    glib.export('GLITE_LOCATION_TMP');

    glib.check_dir(os.environ['GLITE_LOCATION_VAR'])
    glib.check_dir(os.environ['GLITE_LOCATION_LOG'])
    glib.check_dir(os.environ['GLITE_LOCATION_TMP'])

    glib.export('GPT_LOCATION')
    glib.export('GLOBUS_LOCATION')
    glib.export('CATALINA_HOME')
    glib.export('CATALINA_BASE',os.environ['CATALINA_HOME'])
    glib.export('JAVA_HOME')

    glib.addEnvPath("PATH","%s/bin:%s/bin:%s/externals/bin:%s/bin" \
        % (os.environ['JAVA_HOME'],os.environ['GLOBUS_LOCATION'],os.environ['GLITE_LOCATION'],os.environ['GLITE_LOCATION']))
    glib.addEnvPath("LD_LIBRARY_PATH","%s/lib:%s/externals/lib:%s/lib" % (os.environ['GLOBUS_LOCATION'],os.environ['GLITE_LOCATION'],os.environ['GLITE_LOCATION']))
        
    if params.has_key('voms.db.type'):
        if params['voms.db.type'] == 'oracle':
        
            # Set number of startup connections (only valid for the life of the script)
            if not os.environ.has_key("VOMSORACLE_MIN_CONNECTION"):
                os.environ['VOMSORACLE_MIN_CONNECTION'] = params['voms.db.min.connections']

            # Set number of startup connections (only valid for the life of the script)
            if not os.environ.has_key("VOMSORACLE_MAX_CONNECTION"):
                os.environ['VOMSORACLE_MAX_CONNECTION'] = params['voms.db.max.connections']

            # Set number of startup connections (only valid for the life of the script)
            if not os.environ.has_key("VOMSORACLE_INC_CONNECTION"):
                os.environ['VOMSORACLE_INC_CONNECTION'] = params['voms.db.startup.connections']

            # add oracle libraries to LD_LIBRARY_PATH
            glib.addEnvPath("LD_LIBRARY_PATH", "%s/lib" % params['voms.db.oracle.instantclient.location'])
        
    # MANPATH
    glib.addEnvPath('MANPATH', os.environ['GLOBUS_LOCATION'] + '/man')
    glib.addEnvPath('MANPATH', os.environ['GLITE_LOCATION'] + '/share/man')

    # Set environment
    glib.setUserEnv()

#-------------------------------------------------------------------------------
#  Main program begins here 
#-------------------------------------------------------------------------------

if __name__ == '__main__':

    # The script must be run as root
    if not os.geteuid()==0:
        print '"\nThis script must be run as root\n'
        sys.exit(1)
    
    # Get an instance of the library class
    glib = gLib()
    glib.verbose = 0
    verboseCLI = 0
    
    # ------------------------------------------------------------
    # Load parameters
    # ------------------------------------------------------------
    
    # Get an instance of the ConfigParams class
    params = ConfigParams()
    
    # load default parameters
    loadDefaultParameters(params)
        
    try:
        opts, args = glib.getopt(sys.argv[1:], 'chv', \
         ['checkconf', 'help', 'version', 'start', 'stop', \
          'status', 'configure', 'remove','remove-db',\
          'vo'])
                                                            
        for o, a in opts:
            
            if o == "--verbose":
                verboseCLI = 1
                glib.verbose = 1
                
            if o == "--siteconfig":
                params['site.config.url'] = a
                break
                
    except getopt.GetoptError:
        pass

    params['glib'] = glib

    configFiles = ["%s/../glite-rgma-common.cfg.xml" % glib.getScriptPath(), 
                   "%s/../glite-rgma-servicetool.cfg.xml" % glib.getScriptPath(), 
                   "%s/../glite-voms-server.cfg.xml" % glib.getScriptPath()]
        
    if glib.loadConfiguration(configFiles,params):
        glib.printErrorMessage("Problems while loading configuration file.")
        sys.exit(1)
    
    glib.set_params(params)

    # ------------------------------------------------------------    
    # set verbosity
    # ------------------------------------------------------------    
    verbose = 0
    if verboseCLI:
        verbose = 1
    else:
        if params.has_key('glite.installer.verbose'):
            if params['glite.installer.verbose'] == "true":
                verbose = 1
    
    glib.verbose = verbose
  
    # ------------------------------------------------------------
    # Set up the environment
    # ------------------------------------------------------------
    set_env()
          
    # ------------------------------------------------------------          
    # Instantiate the service classes
    # ------------------------------------------------------------
    service = glite_voms(params)
    service.verbose = verbose
    service.tomcat.verbose = verbose
    service.mysql.verbose = verbose
        
    rgmaServicetool = gliteRgmaServicetool()
    rgmaServicetool.verbose = verbose

    # ------------------------------------------------------------
    # Get and check command line parameters
    # ------------------------------------------------------------
    start     = 0
    stop      = 0
    configure = 0
    status    = 0
    check     = 0
    remove    = 0
    
    onlyVo    = 0
    
    removedb  = 0
    commandLineError = 0

    try:
        opts, args = glib.getopt(sys.argv[1:], 'chv', \
            ['checkconf', 'help', 'version', 'start', \
             'stop', 'status','configure', 'remove-db', \
             'remove', 'vo='])
    except getopt.GetoptError:
        commandLineError = 1
    
    if commandLineError==0:
        for o, a in opts:
        
            if o in ("-h", "--help"):
                service.usage()
                sys.exit(0)
            
            if o in ("-v", "--version"):
                service.showVersion()
                sys.exit(0)
            
            if o in ("-c", "--checkconf"):
                check = 1
            
            if o == "--start":
                start = 1
                
            if o == "--stop":
                stop = 1
                 
            if o == "--status":
                status = 1
            
            if o == "--configure":
                configure = 1
                
            if o == "--remove":
                remove = 1

            if o == "--remove-db":
                removedb = 1
                
            if o == "--vo":
                onlyVo = 1
                voName = a
                
    # ------------------------------------------------------------
    # No relevant option given
    # ------------------------------------------------------------
    if (configure==0) and (start==0) and (stop==0) and (status==0) and (remove==0) and (check==0):
        commandLineError = 1

    # ------------------------------------------------------------
    # More than one relevant option given
    # ------------------------------------------------------------
    sumOptions = start+stop+status+configure+check+remove
    if sumOptions>1:
        commandLineError = 1

    # ------------------------------------------------------------          
    # command line error
    # ------------------------------------------------------------
    if commandLineError==1:
        service.usage("")
        sys.exit(0)        

    # ------------------------------------------------------------          
    # check all values
    # ------------------------------------------------------------
    if check==1 and onlyVo==0:
        if service.showConfiguration():
            sys.exit(0)
        else:
            sys.exit(1)
    
    # ------------------------------------------------------------          
    # check vo values
    # ------------------------------------------------------------
    if check==1 and onlyVo==1:
        if service.showConfigurationForOneVo(voName):
            sys.exit(0)
        else:
            sys.exit(1)
    
    # ------------------------------------------------------------          
    # configure all
    # ------------------------------------------------------------
    if configure==1 and onlyVo==0:
        
        # Display the parameters if verbose output
        if verbose:
            glib.print_params(params)
        
        # Configure the service
        return_value = service.configure()
        if return_value == 0:
            print "\nThe %s configuration was successfully completed\n" % service.friendly_name
            print "You can start now the VOMS server via the option --start"
            glib.registerService()
            sys.exit(0)
        elif return_value == 2:
            print "\nThe %s configuration was completed, but one or more warnings" % service.friendly_name
            print "were issued. Please revise them and re-run the script"
            print "or configure and start the %s manually\n" % service.friendly_name
            sys.exit(2)
        else:
            print "\nAn unrecoverable error occurred while configuring the %s service" \
                % service.friendly_name
            sys.exit(1)

    # ------------------------------------------------------------          
    # configure vo
    # ------------------------------------------------------------
    if configure==1 and onlyVo==1:
        
        if service.configureVo(voName) == 0:
            print "\nThe VO '%s' was sucessfully configured for VOMS\n" % voName
            print "You can start now the VO via the option --start --vo=%s" % voName
            sys.exit(0)
        else:
            print "\nAn error occured while configuring VO '%s' for VOMS\n" % voName
            sys.exit(1)

    # ------------------------------------------------------------          
    # start all
    # ------------------------------------------------------------
    if start==1 and onlyVo==0:
        if service.start() == 0:
            print "\nThe %s was successfully started\n" % service.friendly_name
            sys.exit(0)
        else:
            print "\nAn error occurred while starting the %s service" \
                % service.friendly_name
            sys.exit(1)

    # ------------------------------------------------------------          
    # start vo
    # ------------------------------------------------------------
    if start==1 and onlyVo==1:
        if service.startVo(voName) == 0:
            print "\nThe VO '%s' %s was successfully started\n" % (voName, service.friendly_name)
            sys.exit(0)
        else:
            print "\nAn error occurred while starting the VO '%s' %s service" \
                % (voName, service.friendly_name)
            sys.exit(1)
        
    # ------------------------------------------------------------          
    # stop all
    # ------------------------------------------------------------
    if stop==1 and onlyVo==0:
        if service.stop() == 0:
            print "The %s was successfully stopped\n" % service.friendly_name
            sys.exit(0)
        else:
            print "An error occurred while stopping the %s service" \
                % service.friendly_name
            sys.exit(1)

    # ------------------------------------------------------------          
    # stop vo
    # ------------------------------------------------------------
    if stop==1 and onlyVo==1:
        if service.stopVo(voName) == 0:
            print "\nThe VO '%s' %s was successfully stopped\n" % (voName, service.friendly_name)
            sys.exit(0)
        else:
            print "\nAn error occurred while stopping the VO '%s' %s service" \
                % (voName, service.friendly_name)
            sys.exit(1)

    # ------------------------------------------------------------          
    # status all
    # ------------------------------------------------------------
    if status==1 and onlyVo==0:
        retval = service.status()
        sys.exit(retval)

    # ------------------------------------------------------------          
    # status vo
    # ------------------------------------------------------------
    if status==1 and onlyVo==1:
        retval = service.statusVo(voName)
        sys.exit(retval)

    # ------------------------------------------------------------          
    # remove vo
    # ------------------------------------------------------------
    if remove==1 and onlyVo==1:
     
        if service.removeVo(voName,removedb) == 0:
            print "\nThe vo '%s' was sucessfully removed from VOMS\n" % voName
            sys.exit(0)
        else:
            print "\nAn error occured while removing vo '%s' from VOMS\n" % voName
            sys.exit(1)

    else:
        # error in command line
        service.usage("")
        sys.exit(0)  

